CREATE OR REPLACE PROCEDURE DIST_AUTO_PRAT
(
   P_NRDISTCARD IN NUMBER,   
   P_NRSEQPRATCAR IN NUMBER,
   P_TIPO_DIST_AUTO IN VARCHAR2   
)
IS
   /*Dados do prato no cardpio*/   
   V_CDFILIAL VARCHAR2(4);
   V_CDSERVFILI VARCHAR2(5);
   V_CDCLIENTE VARCHAR2(21);
   V_CDDESTFILI VARCHAR2(3);
   V_CDFILAUXPRAT VARCHAR2(4);   
   V_CDPRATO VARCHAR2(10);
   V_VRPECONSCARD FLOAT;
   V_QTPORPESSOA FLOAT; 
   V_QTEXTRA FLOAT;
   
   /*============================*/
   /*=====  VARIVEIS DE CONTROLE  =====*/
   /*============================*/   
   V_CONF_A_UTILIZAR INT;
   
   V_INDICE INT;
   
   /*Chave da embalagem na distribuio do cardpio*/
   V_NREMBDISTCARD NUMBER;
   V_NREMBDISTCARD_J_EXISTENTE NUMBER;
   
   /*Sequencial da novo cdigo*/   
   V_SEQUENCIAL VARCHAR2(20);
   
   /*Valores totais a distribuir*/
   V_TOTAL_COMENSAIS FLOAT;
   V_TOTAL_A_DISTRIBUIR FLOAT;
   
   /*Quantidade total e quantidade de embalagens completas*/
   V_QTDE_EMB FLOAT;
   V_QTDE_EMB_COMP FLOAT;
   V_RESTO FLOAT;
   
   /*Comensais correspondentes*/
   V_COMENSAIS FLOAT;
   V_C FLOAT;
   
   /*Amonstra na embalagem*/
   V_EXTRA_EMB FLOAT;
   V_A FLOAT;
   V_RESTO_A FLOAT;   
   V_QTEMB_A_COMP FLOAT;
   
   /*Quantidade do prato alocada na embalagem*/
   V_QTDE_PRAT FLOAT;
   V_QP FLOAT;      
   V_RESTO_PRAT FLOAT;   
   V_QTEMB_PRAT_COMP FLOAT;
   
   /*Fator utilizado como peso para determinao da prioridade da embalagem*/   
   /*Corresponde a 2 - (quantidade ocupada da embalagem incompleta) */
   V_FATOR_QTDE_N_APROVEITADA FLOAT;
   V_ORDEM_PREFERENCIA FLOAT;
   
   /*Embalagem com o menor valor de ordem (preferida)*/
   V_CDCONTAINER_ESCOLHIDA VARCHAR2(5);
   V_VRQTDEPRATEMB_ESCOLHIDA FLOAT;
   V_ORDEM_PREFERENCIA_ESCOLHIDA FLOAT;
   
   /*Espao no aproveitado da embalagem incompleta*/
   V_DISPONIVEL FLOAT;
   /*======================================*/
   /*=====  CURSOR DE EMBALAGENS A CONSIDERAR  =====*/
   /*                                                                                   */
   /* A varivel V_CONF_A_UTILIZAR controla a execuo          */
   /*======================================*/
   V_CDCONTAINER VARCHAR2(5);
   V_VRMINCONTAINER FLOAT;
   V_NRORDEM NUMBER;   
   V_VRQTDEPRATEMB FLOAT;
   
   CURSOR C_EMBALAGENS
   IS
      /*Relaes com embalagens existentes para o prato*/
      SELECT R.CDCONTAINER, NULL AS VRMINCONTAINER,
                  999999 AS ORDEM,
                  R.VRQTDEPRATEMB
      FROM RELPRATEMB R
      WHERE R.CDFILAUXPRAT = V_CDFILAUXPRAT
           AND R.CDPRATO = V_CDPRATO
           AND V_CONF_A_UTILIZAR = 0
      
      UNION
      
      /*Restries a nvel de servio*/
      SELECT I.CDCONTAINER, I.VRMINCONTAINER,
                  CASE WHEN I.VRMINCONTAINER IS NULL THEN 999999 ELSE I.NRORDEM END AS ORDEM,
                  REL.VRQTDEPRATEMB
        FROM  RESTEMBTPPRAT R                  
                  INNER JOIN ITRESTEMBTPPRAT I ON I.NRRESTEMBTPPRAT = R.NRRESTEMBTPPRAT                  
                  INNER JOIN RELPRATEMB REL ON REL.CDFILAUXPRAT = V_CDFILAUXPRAT AND
                                                                   REL.CDPRATO = V_CDPRATO AND
                                                                   REL.CDCONTAINER = I.CDCONTAINER                                                               
        WHERE R.CDFILIAL = V_CDFILIAL
             AND R.CDSERVFILI = V_CDSERVFILI
             AND R.CDCLIENTE IS NULL
             AND V_CONF_A_UTILIZAR = 1
      
      UNION
      
      /*Restries a nivel de ponto de entrega*/
      SELECT I.CDCONTAINER, I.VRMINCONTAINER,
                  CASE WHEN I.VRMINCONTAINER IS NULL THEN 999999 ELSE I.NRORDEM END AS ORDEM,
                  REL.VRQTDEPRATEMB
        FROM  RESTEMBTPPRAT R
                  INNER JOIN ITRESTEMBTPPRAT I  ON I.NRRESTEMBTPPRAT = R.NRRESTEMBTPPRAT                   
                  INNER JOIN RELPRATEMB REL ON REL.CDFILAUXPRAT = V_CDFILAUXPRAT AND
                                                                   REL.CDPRATO = V_CDPRATO AND
                                                                   REL.CDCONTAINER = I.CDCONTAINER                                                               
        WHERE R.CDFILIAL = V_CDFILIAL
             AND R.CDSERVFILI = V_CDSERVFILI
             AND R.CDCLIENTE = V_CDCLIENTE
             AND R.CDDESTFILI = V_CDDESTFILI
             AND V_CONF_A_UTILIZAR = 2
             
      UNION
      
      /*Restries a nivel de prato*/
      SELECT I.CDCONTAINER, I.VRMINCONTAINER,
                  CASE WHEN I.VRMINCONTAINER IS NULL THEN 999999 ELSE I.NRORDEM END AS ORDEM,
                  REL.VRQTDEPRATEMB
      FROM RESTEMBPRAT R
                INNER JOIN ITRESTEMBPRAT I ON I.NRRESTEMBPRAT = R.NRRESTEMBPRAT
                INNER JOIN RELPRATEMB REL ON REL.CDFILAUXPRAT = V_CDFILAUXPRAT AND
                                                                 REL.CDPRATO = V_CDPRATO AND
                                                                 REL.CDCONTAINER = I.CDCONTAINER
      WHERE R.CDFILIAL = V_CDFILIAL
           AND R.CDSERVFILI = V_CDSERVFILI
           AND R.CDFILAUXPRAT = V_CDFILAUXPRAT
           AND R.CDPRATO = V_CDPRATO
           AND V_CONF_A_UTILIZAR = 3
           
      ORDER BY ORDEM;
                  
BEGIN   
   /*======================================*/
   /*===== BUSCA O PRATO DO  CARDAPIO============*/
   /* Garante que o prato est no cardpio informado               */
   /*======================================*/   
   BEGIN
      SELECT C.CDFILIAL, C.CDSERVFILI, CC.CDFILAUXPRAT, CC.CDPRATO, 
             CNF.QTTOTCOMENSAIS, CNF.QTPORPESSOA, CNF.QTTOTEXTRA, CNF.QTTOTPRATO,
             D.CDCLIENTE, D.CDDESTFILI
         INTO V_CDFILIAL, V_CDSERVFILI, V_CDFILAUXPRAT, V_CDPRATO, 
             V_TOTAL_COMENSAIS, V_QTPORPESSOA, V_QTEXTRA, V_TOTAL_A_DISTRIBUIR,
             V_CDCLIENTE, V_CDDESTFILI
      FROM DISTCARD D
           INNER JOIN CARDADIA C ON C.CDFILIAL = D.CDFILIAL AND
                                    C.NRCARDAPIO = D.NRCARDAPIO
           LEFT JOIN CARDACLI CL ON CL.CDFILIAL = C.CDFILIAL AND
                                     CL.NRCARDAPIO = C.NRCARDAPIO AND
                                     CL.CDCLIENTE = D.CDCLIENTE AND
                                     CL.CDDESTFILI = D.CDDESTFILI
           INNER JOIN COMPCARD CC ON CC.CDFILIAL = C.CDFILIAL AND
                                     CC.NRCARDAPIO = C.NRCARDAPIO AND
                                     CC.NRSEQPRATCAR = P_NRSEQPRATCAR                                     
           INNER JOIN CNFDISTCARD CNF ON CNF.NRDISTCARD = D.NRDISTCARD AND
                                         CNF.NRSEQPRATCAR = P_NRSEQPRATCAR          
      WHERE D.NRDISTCARD = P_NRDISTCARD;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         RETURN;
   END;
   
   IF (V_QTPORPESSOA IS NULL) THEN
      RETURN;
   END IF;
   
   IF (V_QTEXTRA IS NULL) THEN
      V_QTEXTRA := 0;
   END IF;
   
   /*======================================*/
   /*=====  DETERMINA EMBALAGENS A CONSIDERAR =====*/
   /*======================================*/   
   V_CONF_A_UTILIZAR := NULL;
   BEGIN
      SELECT MAX(NVL(CONF_A_UTILIZAR, 0))
         INTO V_CONF_A_UTILIZAR
      FROM
      (
         /*Restries a nvel de servio*/
         SELECT 1 AS CONF_A_UTILIZAR
         FROM DUAL
         WHERE EXISTS(SELECT 1 
                        FROM RESTEMBTPPRAT
                        WHERE CDFILIAL = V_CDFILIAL
                             AND CDSERVFILI = V_CDSERVFILI
                             AND CDCLIENTE IS NULL)
      UNION
         /*Restries a nvel de ponto de entrega*/
         SELECT 2 AS CONF_A_UTILIZAR
         FROM DUAL
         WHERE EXISTS(SELECT 1 
                        FROM RESTEMBTPPRAT
                        WHERE CDFILIAL = V_CDFILIAL
                             AND CDSERVFILI = V_CDSERVFILI
                             AND CDCLIENTE = V_CDCLIENTE
                             AND CDDESTFILI = V_CDDESTFILI)
      UNION
         /*Restries a nvel de prato*/
         SELECT 3 AS CONF_A_UTILIZAR
         FROM DUAL
         WHERE EXISTS(SELECT 1
                        FROM RESTEMBPRAT
                        WHERE CDFILIAL = V_CDFILIAL
                             AND CDSERVFILI = V_CDSERVFILI
                             AND CDFILAUXPRAT = V_CDFILAUXPRAT
                             AND CDPRATO = V_CDPRATO)
      ) S;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_CONF_A_UTILIZAR := 0;
   END;
   
   IF V_CONF_A_UTILIZAR IS NULL THEN
      V_CONF_A_UTILIZAR := 0;
   END IF;

   /*======================================*/
   /*=====  CONTAINERS COM O MNIMO INFORMADO  =====*/   
   /*======================================*/   
      
   OPEN C_EMBALAGENS;
      
   LOOP
      
      FETCH C_EMBALAGENS         
        INTO V_CDCONTAINER, V_VRMINCONTAINER, V_NRORDEM, V_VRQTDEPRATEMB;
         
      EXIT WHEN ((C_EMBALAGENS%NOTFOUND) OR  ((V_TOTAL_A_DISTRIBUIR = 0) AND (V_QTEXTRA = 0)) OR (V_VRMINCONTAINER IS NULL));
      
      V_INDICE := 0;
      
      V_QTDE_EMB := 0;
      V_COMENSAIS := 0;
      V_QTDE_PRAT := 0;
      V_EXTRA_EMB := 0;
   
      /*"Preenche" cada embalagem at atingir o mnimo, reduzindo o total a distribuir a cada iterao*/
      LOOP
         EXIT WHEN (V_INDICE = V_VRMINCONTAINER) OR ((V_TOTAL_A_DISTRIBUIR = 0) AND (V_QTEXTRA = 0));         
         /*Nmero de embalagens*/
         V_QTDE_EMB := V_QTDE_EMB + 1;
         
         /*Distribuio do peso do prato*/
         IF (V_TOTAL_A_DISTRIBUIR > 0) THEN
            /*Se embalagem maior que o total do prato a distribuir*/
            IF (V_VRQTDEPRATEMB > V_TOTAL_A_DISTRIBUIR) THEN
               /*Joga todo o prato na embalagem*/
               V_QP := V_TOTAL_A_DISTRIBUIR;
               V_QTDE_PRAT := V_QTDE_PRAT + V_QP;
               V_COMENSAIS := V_COMENSAIS + V_TOTAL_COMENSAIS;
               
                /*Abate do total de comensais e do total a distribuir o valor do prato naquela embalagem*/
               V_TOTAL_A_DISTRIBUIR := 0;
               V_TOTAL_COMENSAIS := 0;
            /*Se embalagem menor ou igual ao valor total do prato*/
            ELSE
               /*Abate do total a distribuir o valor da capacidade da embalagem (em relao ao prato)*/
               V_QP := V_VRQTDEPRATEMB;
               V_QTDE_PRAT := V_QTDE_PRAT + V_QP;
               V_C := ROUND(V_QP / V_QTPORPESSOA, 5);
               IF V_C > V_TOTAL_COMENSAIS THEN
                  V_C := V_TOTAL_COMENSAIS;
               END IF;
               V_COMENSAIS := V_COMENSAIS + V_C;
               
               /*Abate do total de comensais e do total a distribuir o valor do prato naquela embalagem*/
                V_TOTAL_A_DISTRIBUIR := V_TOTAL_A_DISTRIBUIR - V_QP;
                V_TOTAL_COMENSAIS := V_TOTAL_COMENSAIS - V_C;
            END IF;
            
            /*Verifica se h espao na embalagem incompleta para amostra*/
            IF ((V_TOTAL_A_DISTRIBUIR = 0) AND (V_QTEXTRA > 0)) THEN
               V_A := V_VRQTDEPRATEMB - V_QP;
               
               IF (V_A > V_QTEXTRA) THEN
                  V_A := V_QTEXTRA;                  
               END IF;
               
               /*Aloca amostra na embalagem e abate do total de amostra a distribuir*/
               V_EXTRA_EMB := V_EXTRA_EMB + V_A;
               
               V_QTEXTRA := V_QTEXTRA - V_A;
               
            END IF;
         /*Quando somente a amostra estiver sobrando para distribuir*/
         ELSE
            V_A := V_VRQTDEPRATEMB;
               
            IF (V_A > V_QTEXTRA) THEN
               V_A := V_QTEXTRA;
            END IF;
            
            V_EXTRA_EMB := V_EXTRA_EMB + V_A;
            
            V_QTEXTRA := V_QTEXTRA - V_A;
         END IF;

         V_INDICE := V_INDICE + 1;
      END LOOP;

      /*Busca novo sequencial*/               
      NOVO_CODIGO_SC('EMBDISTCARD',V_SEQUENCIAL);
      V_NREMBDISTCARD := TO_NUMBER(V_SEQUENCIAL);

      /*Insere embalagem na distribuio do cardpio*/
      INSERT INTO EMBDISTCARD
      (NRDISTCARD, NREMBDISTCARD, CDCONTAINER, QTCONTAINER, IDCADAUTO)
      VALUES 
      (P_NRDISTCARD, V_NREMBDISTCARD, V_CDCONTAINER, V_QTDE_EMB, 'S');
      
      /*Insere o prato na embalagem na distribuio*/
      INSERT INTO ITEMBDISTCARD
      (NRDISTCARD, NRSEQPRATCAR, NREMBDISTCARD, 
       QTCOMENSAIS, QTPRATO, QTEXTRA, IDCADAUTO, IDPRINCIPAL)
      VALUES
      (P_NRDISTCARD, P_NRSEQPRATCAR, V_NREMBDISTCARD, 
       V_COMENSAIS, V_QTDE_PRAT, V_EXTRA_EMB, 'S', 'S');


   END LOOP;

   CLOSE C_EMBALAGENS;
   
   /*===================================================*/
   /*=====  ESCOLHE O MELHOR CONTAINER PARA DISTRIBUIR O PRATO  =====*/
   /*===================================================*/
   
   LOOP
      V_CDCONTAINER_ESCOLHIDA := 'NDEF';
      V_VRQTDEPRATEMB_ESCOLHIDA  := 0;
      V_ORDEM_PREFERENCIA_ESCOLHIDA := 999999;
      
      
      EXIT WHEN ((V_TOTAL_A_DISTRIBUIR = 0) AND (V_QTEXTRA = 0));
      
      V_COMENSAIS := 0;
      V_QTDE_PRAT := 0;
      V_EXTRA_EMB := 0;
      
      OPEN C_EMBALAGENS;
      
      /*===================================================*/
      /*=====  PERCORRE TODAS AS EMBALAGENS DISPONVEIS =====*/
      /*=====  CALCULANDO A ORDEM DE PREFERNCIA ============*/
      /*===================================================*/
   
      LOOP
      
         FETCH C_EMBALAGENS         
           INTO V_CDCONTAINER, V_VRMINCONTAINER, V_NRORDEM, V_VRQTDEPRATEMB;
         
         EXIT WHEN C_EMBALAGENS%NOTFOUND;
         
         /*Quantidade de embalagens completas*/
         V_QTDE_EMB_COMP := TRUNC((V_TOTAL_A_DISTRIBUIR + V_QTEXTRA) / V_VRQTDEPRATEMB);

         /*Quantidade na embalagem incompleta*/
         V_RESTO := MOD(V_TOTAL_A_DISTRIBUIR + V_QTEXTRA, V_VRQTDEPRATEMB);         
         V_RESTO := ROUND(V_RESTO, 5);

         /*Fator de no aproveitamento da embalagem incompleta*/
         V_FATOR_QTDE_N_APROVEITADA := 1 + (V_VRQTDEPRATEMB - V_RESTO)/V_VRQTDEPRATEMB;
         
         /*Ordem da embalagem na preferncia : quantidade de embalagens + se o resto no for zero, 1 * fator de no aproveitamento*/
         V_ORDEM_PREFERENCIA := V_QTDE_EMB_COMP + CASE V_RESTO WHEN 0 THEN 0 ELSE V_FATOR_QTDE_N_APROVEITADA END;
         
         /*Se a ordem for menor (melhor), armazena os dados nas variveis auxiliares*/
         IF V_ORDEM_PREFERENCIA <= V_ORDEM_PREFERENCIA_ESCOLHIDA THEN            
            V_ORDEM_PREFERENCIA_ESCOLHIDA := V_ORDEM_PREFERENCIA;
            V_CDCONTAINER_ESCOLHIDA := V_CDCONTAINER;
            V_VRQTDEPRATEMB_ESCOLHIDA := V_VRQTDEPRATEMB;
         END IF;

      END LOOP;
            
      CLOSE C_EMBALAGENS;
      
      /*Se no h embalagens a ser escolhida, termina a funo*/
      IF V_CDCONTAINER_ESCOLHIDA = 'NDEF' THEN
         RETURN;
      END IF;
      
      /*Verifica se o prato j est associado  embalagem*/
      /*Verifica se h espao disponvel para alocao de mais quantidade na embalagem*/
      BEGIN
         SELECT D.NREMBDISTCARD, ROUND(R.VRQTDEPRATEMB - MOD(I.QTPRATO + I.QTEXTRA, R.VRQTDEPRATEMB), 5) AS DISPONIVEL
           INTO V_NREMBDISTCARD_J_EXISTENTE, V_DISPONIVEL
           FROM EMBDISTCARD D
               INNER JOIN ITEMBDISTCARD I ON I.NREMBDISTCARD = D.NREMBDISTCARD
               INNER JOIN RELPRATEMB R ON R.CDCONTAINER = D.CDCONTAINER AND
                                          R.CDFILAUXPRAT = V_CDFILAUXPRAT AND
                                          R.CDPRATO = V_CDPRATO
           WHERE D.NRDISTCARD = P_NRDISTCARD
             AND D.CDCONTAINER = V_CDCONTAINER_ESCOLHIDA
             AND I.NRSEQPRATCAR = P_NRSEQPRATCAR;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_NREMBDISTCARD_J_EXISTENTE := NULL;
      END;
      
      /*Escolhida a melhor embalagem, determina a quantidade de embalagens, o quanto do prato est na embalagem */      
      V_QTEMB_PRAT_COMP := TRUNC(V_TOTAL_A_DISTRIBUIR / V_VRQTDEPRATEMB_ESCOLHIDA);      
      V_RESTO_PRAT := ROUND(MOD(V_TOTAL_A_DISTRIBUIR, V_VRQTDEPRATEMB_ESCOLHIDA), 5);      
      
      /*Quantidade do prato na embalagem (parcial, falta o resto que ser tratado a seguir)*/
      V_QP := V_QTEMB_PRAT_COMP * V_VRQTDEPRATEMB_ESCOLHIDA;
      V_QTDE_PRAT := V_QP;
      
      /*Comensais correspondente  quantidade do prato*/
      V_C := ROUND(V_QP/V_QTPORPESSOA, 5);
      IF V_C > V_TOTAL_COMENSAIS THEN
         V_C := V_TOTAL_COMENSAIS;
      END IF;
      V_COMENSAIS := V_C;
      
      /*Determina o quanto de amostra a embalagem ir transportar*/
      V_QTEMB_A_COMP := TRUNC(V_QTEXTRA / V_VRQTDEPRATEMB_ESCOLHIDA);
      V_RESTO_A := ROUND(MOD(V_QTEXTRA, V_VRQTDEPRATEMB_ESCOLHIDA), 5);
      /*Quantidade da amostra na embalagem (parcial, falta o resto que ser tratado a seguir)*/      
      V_EXTRA_EMB := V_QTEMB_A_COMP * V_VRQTDEPRATEMB_ESCOLHIDA;
      
      /*Quantidade de embalagens*/
      V_QTDE_EMB := V_QTEMB_PRAT_COMP + V_QTEMB_A_COMP;

      /*===================================================*/
      /*=====  VERIFICAO DO RESTANTE A DISTRIBUIR ======*/      
      /*===================================================*/
      IF V_NREMBDISTCARD_J_EXISTENTE IS NOT NULL THEN         
         /*Se a disponibilidade  igual  capacidade, ento  uma nova embalagem*/
         IF V_DISPONIVEL = V_VRQTDEPRATEMB_ESCOLHIDA AND P_TIPO_DIST_AUTO <> 'DISTR' THEN
            V_QTDE_EMB := V_QTDE_EMB + 1;
         END IF;
         
         /*Aloca parte ou todo o resto na embalagem incompleta j existente na distribuio*/         
         IF V_DISPONIVEL > 0 AND 
            ((P_TIPO_DIST_AUTO <> 'DISTR') OR 
             ( (P_TIPO_DIST_AUTO = 'DISTR') AND  V_DISPONIVEL <> V_VRQTDEPRATEMB_ESCOLHIDA)) THEN            
            IF V_RESTO_PRAT > V_DISPONIVEL THEN
               V_RESTO_PRAT := V_RESTO_PRAT - V_DISPONIVEL;               
               V_QTDE_PRAT := V_QTDE_PRAT + V_DISPONIVEL;
               
               /*Comensais correspondentes ao resto da diviso*/
               V_C := ROUND(V_DISPONIVEL/V_QTPORPESSOA, 5);
               IF V_C > V_TOTAL_COMENSAIS THEN
                  V_C := V_TOTAL_COMENSAIS;
               END IF;
               V_COMENSAIS :=V_COMENSAIS + V_C;
               
               V_DISPONIVEL := 0;
            ELSE
               V_DISPONIVEL := V_DISPONIVEL - V_RESTO_PRAT;               
               V_QTDE_PRAT := V_QTDE_PRAT + V_RESTO_PRAT;
               
               /*Comensais correspondentes ao resto da diviso*/
               V_C := ROUND(V_RESTO_PRAT/V_QTPORPESSOA, 5);
               IF V_C > V_TOTAL_COMENSAIS THEN
                  V_C := V_TOTAL_COMENSAIS;
               END IF;
               V_COMENSAIS :=V_COMENSAIS + V_C;
               
               V_RESTO_PRAT := 0;
            END IF;
         END IF;
         
         /*Aloca parte ou todo o resto da amostra na embalagem incompleta j existente na distribuio*/         
         IF V_DISPONIVEL > 0 THEN
            IF V_RESTO_A > V_DISPONIVEL THEN
               V_RESTO_A := V_RESTO_A - V_DISPONIVEL;               
               V_EXTRA_EMB := V_EXTRA_EMB + V_DISPONIVEL;
               V_DISPONIVEL := 0;
            ELSE
               V_DISPONIVEL := V_DISPONIVEL - V_RESTO_A;               
               V_EXTRA_EMB := V_EXTRA_EMB + V_RESTO_A;
               V_RESTO_A := 0;
            END IF;
         END IF;
      END IF;
      
      /*Verifica se a soma dos restos preenche uma embalagem*/
      IF V_RESTO_PRAT + V_RESTO_A >= V_VRQTDEPRATEMB_ESCOLHIDA THEN
         /*INCLUI MAIS UMA EMBALAGEM COMPLETA*/
         V_QTDE_EMB := V_QTDE_EMB + 1;
         
         V_QTDE_PRAT := V_QTDE_PRAT + V_RESTO_PRAT;
         
         /*Comensais correspondentes ao resto da diviso*/
         V_C := ROUND(V_RESTO_PRAT/V_QTPORPESSOA, 5);
         IF V_C > V_TOTAL_COMENSAIS THEN
            V_C := V_TOTAL_COMENSAIS;
         END IF;
         V_COMENSAIS :=V_COMENSAIS + V_C;
         
         /*O resto da amostra completa a embalagem at onde der*/
         /*assim o que passar da soma do resto  abatido do resto da amostra a distribuir*/
         V_RESTO_A := V_RESTO_A - (V_VRQTDEPRATEMB_ESCOLHIDA - V_RESTO_PRAT);

         V_EXTRA_EMB := V_EXTRA_EMB + (V_VRQTDEPRATEMB_ESCOLHIDA - V_RESTO_PRAT);
         
         V_RESTO_PRAT := 0;

      END IF;
      
      /*Verifica se a distribuio no  recursiva, inclui uma embalagem incompleta e aloca os restos*/
      IF (P_TIPO_DIST_AUTO = 'DIST') OR (V_QTDE_EMB = 0)  THEN
         /*INCLUI MAIS UMA EMBALAGEM, INCOMPLETA*/
         V_QTDE_EMB := V_QTDE_EMB + 1;         
         
         V_QTDE_PRAT := V_QTDE_PRAT + V_RESTO_PRAT;
                  
         /*Comensais correspondentes ao resto da diviso*/
         V_C := ROUND(V_RESTO_PRAT/V_QTPORPESSOA, 5);
         IF V_C > V_TOTAL_COMENSAIS THEN
            V_C := V_TOTAL_COMENSAIS;
         END IF;
         V_COMENSAIS := V_COMENSAIS + V_C;
         
         V_EXTRA_EMB := V_EXTRA_EMB + V_RESTO_A;      
      
      END IF;
      
      /*Atualiza o total a distribuir, a quantidade de amostra a distribuir e os comensais correspondentes*/
      V_TOTAL_A_DISTRIBUIR := ROUND(V_TOTAL_A_DISTRIBUIR - V_QTDE_PRAT, 5);
      V_TOTAL_COMENSAIS := ROUND(V_TOTAL_COMENSAIS - V_COMENSAIS, 5);
      V_QTEXTRA := ROUND(V_QTEXTRA - V_EXTRA_EMB, 5);
      
      /*Se no houver mais valores a distribuir, joga o restante dos comensais na embalagem*/
      IF V_TOTAL_A_DISTRIBUIR = 0 THEN
         V_COMENSAIS := V_COMENSAIS + V_TOTAL_COMENSAIS;
         V_TOTAL_COMENSAIS := 0;
      END IF;
      
      /*========================================================================*/
      /*=====  INCLUSO DA EMBALAGEM ESCOLHIDA NA DISTRIBUIO DO PRATO ======*/      
      /*=======================================================================*/
            
      
      /*Se prato j est relacionado  embalagem escolhida, atualiza - a com a nova quantidade*/
      IF V_NREMBDISTCARD_J_EXISTENTE IS NULL THEN         
      
         /*Busca novo sequencial*/               
         NOVO_CODIGO_SC('EMBDISTCARD',V_SEQUENCIAL);
         V_NREMBDISTCARD := TO_NUMBER(V_SEQUENCIAL);

         /*Insere embalagem na distribuio do cardpio*/
         INSERT INTO EMBDISTCARD
         (NRDISTCARD, NREMBDISTCARD, CDCONTAINER, QTCONTAINER, IDCADAUTO)
         VALUES
         (P_NRDISTCARD, V_NREMBDISTCARD, V_CDCONTAINER_ESCOLHIDA, V_QTDE_EMB, 'S');
         
         
         /*Insere o prato na embalagem na distribuio*/
         INSERT INTO ITEMBDISTCARD
         (NRDISTCARD, NRSEQPRATCAR, NREMBDISTCARD, 
          QTCOMENSAIS, QTPRATO, QTEXTRA, IDCADAUTO, IDPRINCIPAL)
         VALUES
         (P_NRDISTCARD, P_NRSEQPRATCAR, V_NREMBDISTCARD, 
          V_COMENSAIS, V_QTDE_PRAT, V_EXTRA_EMB, 'S', 'S');
         
      ELSE

         /*Atualiza a quantidade da embalagem j existente na distribuio*/
         UPDATE EMBDISTCARD
         SET QTCONTAINER = QTCONTAINER + V_QTDE_EMB
         WHERE NREMBDISTCARD = V_NREMBDISTCARD_J_EXISTENTE;
         
         /*Atualiza o prato */
         UPDATE ITEMBDISTCARD
            SET QTCOMENSAIS = QTCOMENSAIS + V_COMENSAIS,
                QTPRATO = QTPRATO + V_QTDE_PRAT,
                QTEXTRA = QTEXTRA + V_EXTRA_EMB,
                IDCADAUTO = 'S'
          WHERE NREMBDISTCARD = V_NREMBDISTCARD_J_EXISTENTE
           AND NRSEQPRATCAR = P_NRSEQPRATCAR;
      
      END IF;

   END LOOP;
   
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('DIST_AUTO_PRAT','5.75.000'); END;
/